"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.ControllerSPA = void 0;

var _promise = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/promise"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/objectSpread"));

var _regenerator = _interopRequireDefault(require("@babel/runtime-corejs2/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/classCallCheck"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/defineProperty"));

var _cluster = _interopRequireDefault(require("cluster"));

var _hapi = _interopRequireDefault(require("hapi"));

var _socket = _interopRequireDefault(require("socket.io"));

var _socket2 = _interopRequireDefault(require("socket.io-redis"));

var _socketioStickySession = _interopRequireDefault(require("socketio-sticky-session"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _logger = _interopRequireDefault(require("@foundationjs/logger"));

var _auth = _interopRequireDefault(require("./controllers/auth"));

var _web = _interopRequireDefault(require("./controllers/web"));

var _hmr = _interopRequireDefault(require("./controllers/hmr"));

var _graphql = _interopRequireDefault(require("./controllers/graphql"));

var _spa = _interopRequireDefault(require("./plugins/spa"));

// Require Cluster
// Require Hapi
// Require SocketIO
// Require JWT
// Require Logger for logs
// Require controllers
// Require plugins to export them
var ControllerSPA = _spa.default;
/**
 * Class Server : initialize a Server instance for a Foundation App
 */

exports.ControllerSPA = ControllerSPA;

var Server =
/**
 * Constructor function
 * @param {object} config - Configuration options (optional)
 */
function Server(_config) {
  var _this = this;

  (0, _classCallCheck2.default)(this, Server);
  (0, _defineProperty2.default)(this, "logger", void 0);
  (0, _defineProperty2.default)(this, "server", void 0);
  (0, _defineProperty2.default)(this, "port", void 0);
  (0, _defineProperty2.default)(this, "usingGraphQL", false);
  (0, _defineProperty2.default)(this, "init",
  /*#__PURE__*/
  function () {
    var _ref = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee(config) {
      var cluster, graphql, _config$log, log;

      return _regenerator.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              cluster = config.cluster, graphql = config.graphql, _config$log = config.log, log = _config$log === void 0 ? {} : _config$log;

              _this.createLogger(log); // Launch the actual init only on forks in clustered mode, and in master if not in clustered mode


              if (!(!cluster || !cluster.sticky || cluster.sticky.size < 2 || !_cluster.default.isMaster)) {
                _context.next = 7;
                break;
              }

              _context.next = 5;
              return _this.slave(config);

            case 5:
              _context.next = 8;
              break;

            case 7:
              _cluster.default.on('exit', function (worker) {
                _this.logger.info("worker ".concat(worker.process.pid, " died"));
              });

            case 8:
              if (!(cluster && cluster.sticky && cluster.sticky.size > 2)) {
                _context.next = 15;
                break;
              }

              if (!_cluster.default.isMaster) {
                _context.next = 12;
                break;
              }

              _context.next = 12;
              return _this.master(config);

            case 12:
              (0, _socketioStickySession.default)({
                proxy: cluster.sticky.proxy,
                // activate layer 4 patching
                header: cluster.sticky.header || 'x-forwarded-for',
                ignoreMissingHeader: true,
                num: cluster.sticky.size || 1
              }, function () {
                _this.logger.level = log.level || 'info';
                return _this.server.listener;
              }).listen(_this.port, function () {
                _this.success({
                  endpoint: "".concat(_this.server.info && _this.server.info.uri, ":").concat(Number(_this.port)),
                  graphql: graphql
                });
              });
              _context.next = 16;
              break;

            case 15:
              _this.success({
                endpoint: _this.server.info.uri,
                graphql: graphql
              });

            case 16:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }());
  (0, _defineProperty2.default)(this, "createLogger", function (log) {
    var logConfig = log || {}; // Append worker id to forks' filename

    if (!_cluster.default.isMaster) {
      logConfig.filename = logConfig.filename && "[".concat(_cluster.default.worker.id, "]_").concat(logConfig.filename);
    } // Prepare the logger


    _this.logger = new _logger.default('Server', logConfig);
  });
  (0, _defineProperty2.default)(this, "logBanner", function () {
    var logger = _this.logger;
    logger.info("Powered by \n  ______                    _       _   _                    _  _____ \n |  ____|                  | |     | | (_)                  | |/ ____|\n | |__ ___  _   _ _ __   __| | __ _| |_ _  ___  _ __        | | (___  \n |  __/ _ \\| | | | '_ \\ / _` |/ _` | __| |/ _ \\| '_ \\   _   | |\\___ \\ \n | | | (_) | |_| | | | | (_| | (_| | |_| | (_) | | | | | |__| |____) |\n |_|  \\___/ \\__,_|_| |_|\\__,_|\\__,_|\\__|_|\\___/|_| |_|  \\____/|_____/");
  });
  (0, _defineProperty2.default)(this, "master",
  /*#__PURE__*/
  function () {
    var _ref3 = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee2(_ref2) {
      var addresses, cluster, graphql, persistence, endpoint, address, server, io;
      return _regenerator.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              addresses = _ref2.addresses, cluster = _ref2.cluster, graphql = _ref2.graphql, persistence = _ref2.persistence, endpoint = _ref2.endpoint;

              _this.logBanner();

              address = endpoint || addresses.endpoint || {};

              if (!endpoint && addresses.endpoint) {
                _this.logger.warn('\'addresses.endpoint\' has been deprecated. Please use \'endpoint\' directly.');
              }

              _this.port = address.port;
              delete address.port;
              address.autoListen = false; // Create the server

              server = _hapi.default.server(address);
              _this.server = server; // Create Socket.IO instance

              io = (0, _socket.default)(server.listener, {
                path: '/foundationjs-socket'
              });

              if (cluster && cluster.redis) {
                io.adapter((0, _socket2.default)(cluster.redis));
              } // Checking if GraphQL will be used


              _this.usingGraphQL = !!graphql && (!!graphql.schema || !!persistence && !!persistence.schema);

            case 12:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function (_x2) {
      return _ref3.apply(this, arguments);
    };
  }());
  (0, _defineProperty2.default)(this, "slave",
  /*#__PURE__*/
  function () {
    var _ref5 = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee4(_ref4) {
      var addresses, endpoint, persistence, cluster, authentication, web, graphql, log, controllers, logConfig, logger, address, server, io, registeredPlugins, secret, schema, pluginsToRegister;
      return _regenerator.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              addresses = _ref4.addresses, endpoint = _ref4.endpoint, persistence = _ref4.persistence, cluster = _ref4.cluster, authentication = _ref4.authentication, web = _ref4.web, graphql = _ref4.graphql, log = _ref4.log, controllers = _ref4.controllers;
              logConfig = log || {};
              logger = _this.logger; // Disable startup logs for workers > 1

              if (!_cluster.default.isMaster) {
                logger.info("Cluster Mode! Instance ".concat(_cluster.default.worker.id, " running"));

                if (_cluster.default.worker.id > 1) {
                  logger.level = 'error';
                }
              }

              if (_cluster.default.isMaster) {
                _this.logBanner(); // Checking if GraphQL will be used


                _this.usingGraphQL = !!graphql && (!!graphql.schema || !!persistence && !!persistence.schema);
              }

              address = endpoint || addresses.endpoint || {};

              if (!endpoint && addresses.endpoint) {
                _this.logger.warn('\'addresses.endpoint\' has been deprecated. Please use \'endpoint\' directly.');
              }

              if (!_cluster.default.isMaster) {
                _this.port = address.port;
                delete address.port;
                address.autoListen = false;
              } // Create the server


              server = _hapi.default.server(address);
              _this.server = server; // Create Socket.IO instance

              io = (0, _socket.default)(server.listener, {
                path: '/foundationjs-socket'
              });

              if (cluster && cluster.redis) {
                io.adapter((0, _socket2.default)(cluster.redis));
              } // Call all controllers


              registeredPlugins = []; // Add Authentication

              logger.info('Initializing Authentications service...');
              secret = authentication ? authentication.secret || '-' : '-';
              _context4.next = 17;
              return (0, _auth.default)(server, authentication || {
                secret: secret
              });

            case 17:
              logger.info('Authentication service initialized successfully'); // Deprecated: Web configuration

              if (!web) {
                _context4.next = 30;
                break;
              }

              logger.warn('Using the built-in Web service is now Deprecated. Please use SPAController or GatsbyController instead'); // Add Web routes (static serving + global redirect)

              logger.info('Initializing Web routes service...');
              _context4.next = 23;
              return (0, _web.default)(server, (0, _objectSpread2.default)({}, web, {
                logger: _this.logger
              }));

            case 23:
              logger.info('Web routes service initialized successfully'); // In dev mode, add Webpack redirect

              if (!(process.env.NODE_ENV === 'development')) {
                _context4.next = 29;
                break;
              }

              logger.info('Initializing HMR service...');
              _context4.next = 28;
              return (0, _hmr.default)(server, addresses.webpack);

            case 28:
              logger.info('HMR service initialized successfully');

            case 29:
              registeredPlugins.push('h2o2', 'vision', 'inert');

            case 30:
              // If a persistence object has been given, initialize the Persistence system
              if (persistence) {
                logger.info('Persistence found! Adding Socket.IO layer...');
                persistence.configureIO({
                  io: io
                });
                logger.info('Socket.IO layer added successfully.');
              } // If a graphql object has been given, initialize the GraphQL engine


              if (!graphql) {
                _context4.next = 43;
                break;
              }

              logger.info('GraphQL config found! Adding GraphQL engine...'); // Create GraphQL schema

              schema = graphql.schema || persistence && persistence.schema;

              if (persistence && persistence.schema && schema === persistence.schema) {
                logger.info('GraphQL is using the Persistence instance as schema');
              } else if (schema === graphql.schema) {
                logger.info('Using custom GraphQL schema provided');
              }

              if (schema) {
                _context4.next = 40;
                break;
              }

              logger.error('You must provide a GraphQL schema to use GraphQL! ' + 'Either use `graphql.schema` or provide a Persistence instance.');
              logger.error('Failed to add GraphQL engine, GraphQL will not work.');
              _context4.next = 43;
              break;

            case 40:
              _context4.next = 42;
              return (0, _graphql.default)(server, (0, _objectSpread2.default)({}, graphql, {
                graphqlOptions: function () {
                  var _graphqlOptions = (0, _asyncToGenerator2.default)(
                  /*#__PURE__*/
                  _regenerator.default.mark(function _callee3(req) {
                    var res;
                    return _regenerator.default.wrap(function _callee3$(_context3) {
                      while (1) {
                        switch (_context3.prev = _context3.next) {
                          case 0:
                            if (!graphql.graphqlOptions) {
                              _context3.next = 6;
                              break;
                            }

                            _context3.next = 3;
                            return graphql.graphqlOptions(req);

                          case 3:
                            _context3.t0 = _context3.sent;
                            _context3.next = 7;
                            break;

                          case 6:
                            _context3.t0 = {};

                          case 7:
                            res = _context3.t0;
                            return _context3.abrupt("return", {
                              context: (0, _objectSpread2.default)({}, res, {
                                authentication: {
                                  create: function create(payload, options) {
                                    return _jsonwebtoken.default.sign(payload, secret, options);
                                  },
                                  get: function get() {
                                    return req.auth.credentials;
                                  }
                                }
                              }),
                              schema: schema
                            });

                          case 9:
                          case "end":
                            return _context3.stop();
                        }
                      }
                    }, _callee3, this);
                  }));

                  function graphqlOptions(_x4) {
                    return _graphqlOptions.apply(this, arguments);
                  }

                  return graphqlOptions;
                }()
              }));

            case 42:
              logger.info('GraphQL engine added successfully');

            case 43:
              pluginsToRegister = []; // Check if any custom controllers need to be initialized

              if (!(controllers && controllers.length)) {
                _context4.next = 50;
                break;
              }

              controllers.forEach(function (c) {
                if (c.plugins && c.plugins.length) {
                  c.plugins.forEach(function (plugin) {
                    var name = null; // Check if the plugin is using the ".plugin" definition

                    if (plugin.plugin) {
                      // If it is, check if we are embedding it in a plugin/options object
                      if (plugin.plugin.plugin) {
                        name = plugin.plugin.plugin.pkg.name;
                      } else {
                        name = plugin.plugin.pkg.name;
                      }
                    } else {
                      name = plugin.pkg.name;
                    }

                    if (registeredPlugins.indexOf(name) < 0) {
                      pluginsToRegister.push(plugin);
                      registeredPlugins.push(name);
                    }
                  });
                }
              }); // Register all necessary plugins

              _context4.next = 48;
              return _promise.default.all(pluginsToRegister.map(function (p) {
                return server.register(p);
              }));

            case 48:
              _context4.next = 50;
              return _promise.default.all(controllers.map(function (c, i) {
                var plugin = {
                  name: "foundationjs-".concat(i),
                  register: c.initialize
                };
                return server.register(plugin);
              }));

            case 50:
              _context4.next = 52;
              return server.start();

            case 52:
              // Re-enable logs for workers > 1
              if (!_cluster.default.isMaster && _cluster.default.worker.id > 1 && logConfig.level) {
                logger.level = logConfig.level;
              }

              logger.info('All initialization successful');

            case 54:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    return function (_x3) {
      return _ref5.apply(this, arguments);
    };
  }());
  (0, _defineProperty2.default)(this, "success", function (_ref6) {
    var endpoint = _ref6.endpoint,
        graphql = _ref6.graphql;

    if (_cluster.default.isMaster) {
      _this.logger.info("App running at: ".concat(endpoint));

      if (_this.usingGraphQL && !!graphql) {
        _this.logger.info("GraphQL endpoint at: ".concat(endpoint).concat(graphql.graphql));

        if (graphql.enableGraphiQL && !!graphql.graphiql) {
          _this.logger.info("Graph(i)QL dashboard at: ".concat(endpoint).concat(graphql.graphiql));
        }
      }
    }
  });

  if (_config) {
    this.init(_config);
  }
}
/**
 * Init function
 * @param {object} config - Configuration options
 */
;

exports.default = Server;