"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime-corejs2/regenerator"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/objectSpread"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/asyncToGenerator"));

var _inert = _interopRequireDefault(require("inert"));

var _h2o = _interopRequireDefault(require("h2o2"));

var _vision = _interopRequireDefault(require("vision"));

var _hapiReactViews = _interopRequireDefault(require("hapi-react-views"));

var _logger = _interopRequireDefault(require("@foundationjs/logger"));

var logger = new _logger.default('SPAController');
/*
 * The SPA Controller exposes a single-page app on a given path,
 * statically built using Webpack, or served by Webpack-Dev-Server in development
 */

var ControllerSPA = function ControllerSPA(options) {
  var path = options.path,
      forceStatic = options.forceStatic,
      hmr = options.hmr,
      statics = options.statics,
      views = options.views;

  function initialize(_x) {
    return _initialize.apply(this, arguments);
  }

  function _initialize() {
    _initialize = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee(server) {
      var production, staticPath;
      return _regenerator.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              // The SPA Controller is made of two parts: a static serving system for resources,
              // and a view system for rendering the initial page(s)
              // In development, we can use options.forceStatic to run in production-like mode
              production = process.env.NODE_ENV === 'production' || forceStatic;
              logger.info("Initializing SPA Controller on path ".concat(options.path, " (Environment: ").concat(process.env.NODE_ENV || 'development', ", Mode: ").concat(production ? 'static' : 'webpack', ")"));

              if (views) {
                /* Configure view engine to use React */
                server.views({
                  engines: (0, _objectSpread2.default)({
                    jsx: _hapiReactViews.default,
                    js: _hapiReactViews.default
                  }, views.engines),
                  path: views.dir,
                  defaultExtension: 'js'
                });
                /* Register view routes */

                if (views.paths) {
                  (0, _keys.default)(views.paths).forEach(function (p) {
                    var view = views.paths[p];
                    var viewPath = path + (path.endsWith('/') ? '' : '/') + (p.startsWith('/') ? p.slice(1) : p);
                    server.route({
                      method: 'GET',
                      path: viewPath,
                      handler: {
                        view: view
                      }
                    });
                    logger.info("Registered view '".concat(view, "' on path ").concat(viewPath));
                  });
                } else if (logger) {
                  logger.warn('View Engine initialized with no views');
                }
              }

              staticPath = path + (path.endsWith('/') ? '' : '/') + (statics.path.startsWith('/') ? statics.path.slice(1) : statics.path);

              if (staticPath.endsWith('/')) {
                staticPath = staticPath.slice(0, -1);
              }

              if (production) {
                /* Route for serving static files */
                server.route({
                  method: 'GET',
                  path: "".concat(path).concat(statics.path, "/{param*}"),
                  handler: {
                    directory: {
                      path: staticPath
                    }
                  }
                });
                logger.info("Serving static files from ".concat(statics.dir, " on path ").concat(staticPath, "/"));
              } else {
                /* Webpack Proxy */
                server.route({
                  method: 'GET',
                  path: "".concat(staticPath, "/{webpackPath*}"),
                  handler: {
                    proxy: {
                      host: hmr.endpoint,
                      port: hmr.port,
                      passThrough: true
                    }
                  }
                });
                logger.info("Webpack proxy established to address ".concat(hmr.endpoint, ":").concat(hmr.port));
              }

              logger.info("SPA site served on ".concat(options.path));

            case 7:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));
    return _initialize.apply(this, arguments);
  }

  return {
    initialize: initialize,
    plugins: [_vision.default, _inert.default, _h2o.default]
  };
};

var _default = ControllerSPA;
exports.default = _default;
module.exports = exports.default;